<template>
  <gree-view>
    <gree-page class="page-sleep">
      <gree-header :left-options="{ showBack: false }" :right-options="{ showMore: false }">
        <span v-text="'编辑DIY睡眠曲线'" />
        <i class="iconfont iconfont-fanhui" slot="overwrite-left" @click="goBack" />
        <a class="save" slot="right" @click="saveDIY">保存</a>
      </gree-header>
      <div class="chart-block">
        <div class="sleep-chart" ref="mychart" />
      </div>
    </gree-page>
  </gree-view>
</template>

<script>
import echarts from 'echarts';
import { Header, Toast, Switch, List, Item, RadioList } from 'gree-ui';
import { mapState, mapActions, mapMutations } from 'vuex';
import { sendDataToDevice } from '@PluginInterface';
import { types } from '@/store/types';
import sleepCurveChartConfig from '@/mixins/sleepCurveChart';

export default {
  components: {
    [Header.name]: Header,
    [Switch.name]: Switch,
    [Toast.name]: Toast,
    [List.name]: List,
    [Item.name]: Item,
    [RadioList.name]: RadioList
  },
  mixins: [sleepCurveChartConfig],
  data() {
    return {
      disableUpdate: false,
      position: [],
      checkTimer: null,
      data: [
        [1, 26],
        [2, 27],
        [3, 28],
        [4, 28],
        [5, 28],
        [6, 28],
        [7, 26],
        [8, 26]
      ],
      temToVal: {
        16: [160, 0],
        17: [170, 0],
        18: [180, 0],
        19: [190, 0],
        20: [200, 0],
        21: [210, 0],
        22: [220, 0],
        23: [230, 0],
        24: [240, 0],
        25: [250, 0],
        26: [4, 1],
        27: [14, 1],
        28: [24, 1],
        29: [34, 1],
        30: [44, 1]
      }
    };
  },
  computed: {
    ...mapState('control', {
      dataObject: state => state.dataObject,
      SwhSlp: state => state.dataObject.SwhSlp,
      statueJson2: state => state.devOptions.statueJson2,
      SlpMod: state => state.dataObject.SlpMod,
      SmartSlpMod: state => state.dataObject.SmartSlpMod,
      SmartSlpModEx: state => state.dataObject.SmartSlpModEx,
      functype: state => state.dataObject.functype,
      Lig: state => state.dataObject.Lig,
      AntiDirectBlow: state => state.dataObject.AntiDirectBlow,
      Mod: state => state.dataObject.Mod,
      Pow: state => state.dataObject.Pow,
      mac: state => state.mac
    }),
    myChart() {
      let dom = this.$refs.mychart;
      return echarts.init(dom);
    },
    temArr() {
      return this.data.map(item => {
        return item[1];
      });
    },
    clientHeight() {
      return document.body.clientHeight;
    },
    clientWidth() {
      return document.body.clientWidth;
    }
  },
  created() {},
  mounted() {
    //mqtt三期设备更新智眠曲线
    this.$bus.$on('mqttUpDate', Obj => {
      this.updateLocal(Obj);
    });
    if (this.mqttVer < 2) {
      this.getSlpVal(); // 轮询
      this.checkTimer = setInterval(() => {
        this.getSlpVal(); // 轮询
      }, 8000);
    }
    this.updateEchart();
    this.$nextTick(() => {
      this.updateLocal(this.dataObject);
    });
  },
  destroyed() {
    clearInterval(this.checkTimer);
  },
  methods: {
    ...mapActions({
      sendData: types.SEND_DATA,
      sendCtrl: types.SEND_CTRL
    }),
    ...mapMutations('control', {
      setChartData: 'SET_CHART_DATA',
      setDataObject: 'SET_DATA_OBJECT'
    }),
    ...mapMutations({
      setCheckObject: types.SET_CHECK_OBJECT
    }),
    async changeDataObject(obj, hasToast = false) {
      console.log('changeDataObject====', obj);
      // clearInterval(this.checkTimer);
      // this.waitCallBack += 1;
      this.disableUpdate = true;
      // this.getSlpTimer = false;
      const sendObj = { ...obj };
      const opt = Object.keys(sendObj);
      const p = Object.values(sendObj);
      const json = JSON.stringify({
        mac: this.mac,
        t: 'cmd',
        opt,
        p
      });
      console.log('changeDataObject====opt,p', opt, p);
      this.setDataObject(sendObj);
      this.setCheckObject(sendObj);
      const res = await sendDataToDevice(this.mac, json, true);
      const { r } = JSON.parse(res);
      console.log(JSON.parse(res));
      if (r === 200 && hasToast) {
        this.$toast(hasToast);
      }
      // this.waitCallBack -= 1;
      // if (!this.waitCallBack) {
      //   this.checkTimer = setInterval(() => {
      //     this.getSlpVal(); // 轮询
      //   }, 5000);
      // }
      this.disableUpdate = false;
    },
    async getSlpVal() {
      if (this.disableUpdate) return;
      this.getSlpTimer = true;
      // data
      const cols = [
        'SwhSlp',
        'SlpMod',
        'SmartSlpMod',
        'SmartSlpModEx',
        'Slp1L1',
        'Slp1H1',
        'Slp1L2',
        'Slp1H2',
        'Slp1L3',
        'Slp1H3',
        'Slp1L4',
        'Slp1H4',
        'Slp1L5',
        'Slp1H5',
        'Slp1L6',
        'Slp1H6',
        'Slp1L7',
        'Slp1H7',
        'Slp1L8',
        'Slp1H8',
        'Pow'
      ];
      const statueJson = JSON.stringify({
        mac: this.mac,
        t: 'status',
        cols
      });
      const res = await sendDataToDevice(this.mac, statueJson, false);
      const _res = JSON.parse(res);
      const dataObject = {};
      _res.forEach((item, index) => {
        dataObject[cols[index]] = item;
      });
      console.log('智眠页-=--------------xxxx', dataObject);

      if (this.disableUpdate || !this.getSlpTimer || this.waitCallBack) return;
      this.updateLocal(dataObject);
      this.updateDataObject(dataObject);
    },
    updateLocal(dataObject) {
      for (let i = 1; i <= 8; i += 1) {
        const typeKey = `Slp1H${i}`;
        const valueKey = `Slp1L${i}`;
        let result;
        if (dataObject[typeKey]) {
          result = Math.round((dataObject[valueKey] - 4) / 10 + 26);
        } else {
          result = Math.round(dataObject[valueKey] / 10);
        }
        this.data[i - 1][1] = result;
      }
      this.updateEchart();
    },
    updateEchart() {
      this.myChart.setOption(this.opt);

      const option = {
        graphic: echarts.util.map(this.data, (item, dataIndex) => {
          return {
            type: 'circle',
            position: this.myChart.convertToPixel('grid', item),
            shape: {
              cx: 0,
              cy: 0,
              r: (25 * this.clientHeight) / 736
            },
            invisible: true,
            draggable: true,
            ondrag: echarts.util.curry(this.onPointDragging, dataIndex),
            onmousemove: echarts.util.curry(this.showTooltip, dataIndex),
            onmouseout: echarts.util.curry(this.hideTooltip, dataIndex),
            z: 100
          };
        })
      };

      option.graphic.push({
        type: 'text',
        // right: 'center',
        origin: [100, 0],
        style: {
          text: '舒适范围',
          x: this.clientWidth * 0.46,
          y: this.clientHeight * 0.18,
          font: `20px Microsoft YaHei`,
          fill: 'rgba(152,152,152,0.2)'
        }
      });

      this.myChart.setOption(option);

      window.addEventListener('resize', this.updatePosition);

      this.myChart.on('dataZoom', this.updatePosition);

      this.updatePosition(true);
    },
    updatePosition(update) {
      if (!this.position.length || update) {
        this.position = echarts.util.map(this.data, item => {
          return this.myChart.convertToPixel('grid', item);
        });
      }
      this.myChart.setOption({
        graphic: echarts.util.map(this.data, (item, dataIndex) => {
          return {
            position: this.position[dataIndex]
          };
        })
      });
    },
    onPointDragging(dataIndex, evt) {
      const position = [evt.offsetX, evt.offsetY];
      let data = Math.round(this.myChart.convertFromPixel('grid', position)[1]);

      data > 30 && (data = 30);
      data < 16 && (data = 16);

      // if (data === this.data[dataIndex][1]) return;

      this.$set(this.data[dataIndex], 1, data);

      this.position[dataIndex] = this.myChart.convertToPixel('grid', this.data[dataIndex]);

      this.myChart.setOption({
        series: [
          {
            id: 'a',
            type: 'line',
            data: this.data,
            symbolSize: 15,
            itemStyle: {
              color: '#77a7e0'
            },
            lineStyle: {
              shadowColor: 'rgba( 152,152,152 ,0.3)',
              shadowBlur: 5,
              shadowOffsetY: 11,
              color: new echarts.graphic.LinearGradient(0, 0, 1, 0, [
                { offset: 0, color: '#77a7e0' },
                { offset: 1, color: '#a4a3e8' }
              ]),
              width: 3
            }
          }
        ]
      });

      this.updatePosition();
      this.disableUpdate = true;
    },
    showTooltip(dataIndex) {
      this.myChart.dispatchAction({
        type: 'showTip',
        seriesIndex: 0,
        dataIndex
      });
      let dom = document.getElementsByClassName('sleep-chart')[0].childNodes[1];
      dom.style.backgroundColor = 'transparent';
    },
    hideTooltip() {
      this.myChart.dispatchAction({
        type: 'hideTip'
      });
    },
    saveDIY() {
      let setData = {
        SwhSlp: 1,
        SlpMod: 3
      };
      const slpDiy = {};
      this.temArr.forEach((item, index) => {
        const keyL = `Slp1L${index + 1}`;
        const keyH = `Slp1H${index + 1}`;
        const opt = this.temToVal[this.temArr[index]];
        [slpDiy[keyL], slpDiy[keyH]] = opt;
      });
      window.storage.set('slpDiy', slpDiy);
      setData = {
        ...setData,
        ...slpDiy
      };
      this.changeDataObject(setData, '保存成功');
    },
    /**
     * @description 返回键
     */
    goBack() {
      this.$router.push({ name: 'Home' });
    }
  }
};
</script>

<style lang="scss">
.page-sleep {
  .gree-header {
    background-color: #ffffff;
  }
  .chart-block {
    height: 1160px;
    background-color: #ffffff;
    .sleep-chart {
      overflow: hidden;
      height: 1100px;
      width: 100%;
      .chart-tooltip {
        font-family: 'Roboto-Light';
        background-color: #ffffff;
        color: #989898;
        width: 140px;
        height: 70px;
        line-height: 70px;
        z-index: 1000;
        border-radius: 86px;
        box-shadow: 6px 6px 10px rgba(0, 0, 0, 0.1);
        text-align: center;
        font-size: 42px;
        position: absolute;
        bottom: 0%;
        right: 50%;
        transform: translateZ(1.1);
        transform: translate(35%, -30%);
        &::after {
          content: '';
          border: none;
          position: absolute;
          font-size: 42px;
          color: #ffffff;
          border-width: 17px;
          top: 100%;
          left: 50%;
          transform: translate(-50%);
          border-color: transparent;
          border-style: solid;
          border-top-color: inherit;
        }
      }
    }
  }
}
</style>
